@@ -27,11 +27,10 @@ use crate::nixpacks::{
     environment::Environment,
     logger::Logger,
     nix::pkg::Pkg,
-    plan::{
-        config::GeneratePlanConfig, generator::NixpacksBuildPlanGenerator, BuildPlan, PlanGenerator,
-    },
+    plan::{generator::NixpacksBuildPlanGenerator, BuildPlan, PlanGenerator},
 };
-use anyhow::Result;
+use anyhow::{bail, Result};
+use nixpacks::plan::generator::GeneratePlanOptions;
 use providers::{
     clojure::ClojureProvider, crystal::CrystalProvider, csharp::CSharpProvider, dart::DartProvider,
     deno::DenoProvider, elixir::ElixirProvider, fsharp::FSharpProvider, go::GolangProvider,
@@ -71,12 +70,12 @@ pub fn get_providers() -> &'static [&'static dyn Provider] {
 pub fn generate_build_plan(
     path: &str,
     envs: Vec<&str>,
-    plan_options: &GeneratePlanConfig,
+    options: &GeneratePlanOptions,
 ) -> Result<BuildPlan> {
     let app = App::new(path)?;
     let environment = Environment::from_envs(envs)?;
 
-    let mut generator = NixpacksBuildPlanGenerator::new(get_providers(), plan_options.clone());
+    let mut generator = NixpacksBuildPlanGenerator::new(get_providers(), options.clone());
     let plan = generator.generate_plan(&app, &environment)?;
 
     Ok(plan)
@@ -85,7 +84,7 @@ pub fn generate_build_plan(
 pub fn create_docker_image(
     path: &str,
     envs: Vec<&str>,
-    plan_options: &GeneratePlanConfig,
+    plan_options: &GeneratePlanOptions,
     build_options: &DockerBuilderOptions,
 ) -> Result<()> {
     let app = App::new(path)?;
@@ -94,6 +93,12 @@ pub fn create_docker_image(
     let mut generator = NixpacksBuildPlanGenerator::new(get_providers(), plan_options.clone());
     let plan = generator.generate_plan(&app, &environment)?;
 
+    if let Some(ref phase) = plan.start_phase {
+        if phase.cmd.is_none() && !build_options.no_error_without_start {
+            bail!("No start command could be found")
+        }
+    }
+
     let logger = Logger::new();
     let builder = DockerImageBuilder::new(logger, build_options.clone());
     builder.create_image(app.source.to_str().unwrap(), &plan, &environment)?;
