@@ -11,8 +11,11 @@ pub struct DockerBuilderOptions {
     pub quiet: bool,
     pub cache_key: Option<String>,
     pub no_cache: bool,
+    pub inline_cache: bool,
+    pub cache_from: Option<String>,
     pub platform: Vec<String>,
     pub current_dir: bool,
+    pub no_error_without_start: bool,
 }
 
 mod cache;
